"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*!
 * module acquisitionservices
 * Copyright(c) 2019 STMicroelectronics
 */
const worker_threads_1 = require("worker_threads");
const index_1 = require("./index");
const stm32cubemonitor_logger_1 = require("@stm32/stm32cubemonitor-logger");
const logger = (0, stm32cubemonitor_logger_1.getLogger)("acquisitionservices");
/**
 * Catch the unexpected error from worker process.
 */
process.on("uncaughtException", function (err) {
    logger.error("PROBE - UNKNOWN STATE: ", err.message);
});
/*
 * Create a new aquisition service
 */
const acquService = new index_1.AcquisitionServices(worker_threads_1.workerData.probeId, worker_threads_1.workerData.settings, {
    protocol: worker_threads_1.workerData.protocol,
    frequency: worker_threads_1.workerData.frequency,
    connectionType: worker_threads_1.workerData.connectionType,
    acquFrequThreshold: worker_threads_1.workerData.acquFrequThreshold
}, worker_threads_1.workerData.sab);
// Event from main thread to acquisition services
if (worker_threads_1.parentPort) {
    worker_threads_1.parentPort.on("message", (messageFromMain) => {
        switch (messageFromMain.cmd) {
            case "connect":
                acquService.connect();
                break;
            case "close":
                acquService.close();
                break;
            case "sendCommand":
                acquService.sendCommand(messageFromMain.args.topic, messageFromMain.args.data);
                break;
            case "exit":
                process.exit();
        }
    });
}
//From acquisition services to main thr
acquService.on("open", function (msg) {
    if (worker_threads_1.parentPort) {
        worker_threads_1.parentPort.postMessage({ cmd: "open", msg: msg });
    }
});
acquService.on("close", function (msg) {
    if (worker_threads_1.parentPort) {
        worker_threads_1.parentPort.postMessage({ cmd: "close", msg: msg });
    }
});
acquService.on("data", function (msg) {
    if (worker_threads_1.parentPort) {
        worker_threads_1.parentPort.postMessage({ cmd: "data", msg: msg });
    }
});
acquService.on("error", function (msg) {
    msg.threadId = worker_threads_1.threadId;
    if (worker_threads_1.parentPort) {
        worker_threads_1.parentPort.postMessage({ cmd: "error", msg: msg });
    }
});
acquService.on("stop", function () {
    if (worker_threads_1.parentPort) {
        worker_threads_1.parentPort.postMessage({ cmd: "stop", msg: "" });
    }
});
