/*!
 * module acquisitionservices
 * Copyright(c) 2019 STMicroelectronics
 */
import { parentPort, workerData, threadId } from "worker_threads";
import { AcquisitionServices } from "./index";
import { getLogger } from "@stm32/stm32cubemonitor-logger";
const logger = getLogger("acquisitionservices");
/**
 * Catch the unexpected error from worker process.
 */
process.on("uncaughtException", function (err) {
  logger.error("PROBE - UNKNOWN STATE: ", err.message);
});

/*
 * Create a new aquisition service
 */
const acquService = new AcquisitionServices(
  workerData.probeId,
  workerData.settings,
  {
    protocol: workerData.protocol,
    frequency: workerData.frequency,
    connectionType: workerData.connectionType,
    acquFrequThreshold: workerData.acquFrequThreshold
  },
  workerData.sab
);
// Event from main thread to acquisition services
if (parentPort) {
  parentPort.on("message", (messageFromMain) => {
    switch (messageFromMain.cmd) {
      case "connect":
        acquService.connect();
        break;
      case "close":
        acquService.close();
        break;
      case "sendCommand":
        acquService.sendCommand(messageFromMain.args.topic, messageFromMain.args.data);
        break;
      case "exit":
        process.exit();
    }
  });
}
//From acquisition services to main thr
acquService.on("open", function (msg) {
  if (parentPort) {
    parentPort.postMessage({ cmd: "open", msg: msg });
  }
});
acquService.on("close", function (msg) {
  if (parentPort) {
    parentPort.postMessage({ cmd: "close", msg: msg });
  }
});
acquService.on("data", function (msg) {
  if (parentPort) {
    parentPort.postMessage({ cmd: "data", msg: msg });
  }
});
acquService.on("error", function (msg) {
  msg.threadId = threadId;
  if (parentPort) {
    parentPort.postMessage({ cmd: "error", msg: msg });
  }
});
acquService.on("stop", function () {
  if (parentPort) {
    parentPort.postMessage({ cmd: "stop", msg: "" });
  }
});
